"use strict";
exports.__esModule = true;
exports._testCreateUseCreateQuery = exports.useCreateQuery = void 0;
var tslib_1 = require("tslib");
var React = (0, tslib_1.__importStar)(require("react"));
var R = React;
var requestIdCount = 0;
var requestIdGenerator = function () {
    requestIdCount += 1;
    return "usecreatequery-".concat(requestIdCount);
};
/**
 * Creates a query from an async getter function.
 *
 * ```ts
 * const query = useCreateQuery(async () => {
 *  const response = await fetch("https://example.com");
 *  return response.json();
 * });
 * ```
 */
var useCreateQuery = function (getter, dependencies) {
    var safeDependencies = dependencies !== null && dependencies !== void 0 ? dependencies : [];
    var requestId = R.useRef(requestIdGenerator()).current;
    var _a = R.useReducer(function (state, action) {
        switch (action.type) {
            case "load":
                return (0, tslib_1.__assign)((0, tslib_1.__assign)({}, state), { isSuccess: false, isError: false, isFetching: false, isLoading: true, isUninitialized: false, startedTimeStamp: Date.now(), refetch: action.payload.refetch });
            case "fetch":
                return (0, tslib_1.__assign)((0, tslib_1.__assign)({}, state), { isLoading: false, isSuccess: false, isError: false, isFetching: true, isUninitialized: false, startedTimeStamp: Date.now(), refetch: action.payload.refetch });
            case "success":
                return (0, tslib_1.__assign)((0, tslib_1.__assign)({}, state), { isLoading: false, isFetching: false, isError: false, isUninitialized: false, isSuccess: true, data: action.payload.data, currentData: action.payload.data, fulfilledTimeStamp: Date.now() });
            case "error":
                return (0, tslib_1.__assign)((0, tslib_1.__assign)({}, state), { isLoading: false, isSuccess: false, isFetching: false, isUninitialized: false, isError: true, error: action.payload.error, fulfilledTimeStamp: Date.now() });
            default:
                return state;
        }
    }, {
        isLoading: true,
        isSuccess: false,
        isError: false,
        isFetching: false,
        refetch: function () { },
        isUninitialized: true,
        currentData: undefined,
        data: undefined,
        error: undefined,
        endpointName: "",
        fulfilledTimeStamp: 0,
        originalArgs: safeDependencies,
        requestId: requestId,
        startedTimeStamp: 0
    }), state = _a[0], dispatch = _a[1];
    var runQuery = function (overrideInitialized) {
        var fetchData = function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
            var data, error_1;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, getter()];
                    case 1:
                        data = _a.sent();
                        dispatch({ type: "success", payload: { data: data } });
                        return [3 /*break*/, 3];
                    case 2:
                        error_1 = _a.sent();
                        dispatch({ type: "error", payload: { error: error_1 } });
                        return [3 /*break*/, 3];
                    case 3: return [2 /*return*/];
                }
            });
        }); };
        dispatch({
            type: overrideInitialized
                ? "fetch"
                : state.isUninitialized
                    ? "load"
                    : "fetch",
            payload: { refetch: function () { return runQuery(true); } }
        });
        fetchData();
    };
    R.useEffect(function () { return runQuery(); }, safeDependencies);
    return state;
};
exports.useCreateQuery = useCreateQuery;
var _testCreateUseCreateQuery = function (react) {
    R = react;
    return exports.useCreateQuery;
};
exports._testCreateUseCreateQuery = _testCreateUseCreateQuery;
//# sourceMappingURL=createQuery.js.map